/*
 * traveling.js
 * Copyright  2007 Tommi Rautava
 * 
 * This file is part of Popomungo.
 *
 * Popomungo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Popomungo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

var pm_Traveling = {

	ROUTE_NAME_REGEXP: /^\s*(.*?)\s*$/,

	CITY_NAME_REGEXP: /^\s*(.*?)\,/i,
	
	BUS_STATION_LINK_ON_CAREER_PAGE_XPATH:
		"/html/body/table[3]/tbody/tr/td[1]/table[2]/tbody/tr/td/table/tbody/tr/td[2]/table[1]/tbody/tr[1]/td[2]/a[1]",
	
	ROUTE_NAME_TARGET_ON_HIGHWAY_LOCALE_PAGE_XPATH:
		'/html/body/table[3]/tbody/tr[1]/td/table/tbody/tr/td[1]/div/br[1]',
		
	LOCALE_ID_REGEXP: /[&?]LocaleID=(\d+)/i,
	
	HOURS_ON_ROUTE_NAME: /[\(\ \/](\d+)h/,
	
	LOCALE_XPATH: '/html/body/table[3]/tbody/tr/td[1]//a[contains(@href, "Locale.asp")]',
	
	LAST_TRANSPORT_ROW_XPATH: "//div[@id='transports']//tr[last()]",
	
	DATE_TIME_AND_CITY_NAME_ON_TRANSPORT_ROW_REGEXP: /(\d+[\.\-\/]\d+[\.\-\/]\d+)\s+(\d+[\.\:]\d+)\s+(.+)/,
	
	FROM_LOCALE_ID_NAME: 'FromLocaleID',
	
	TO_LOCALE_ID_NAME: 'ToLocaleID',
	
	TOUR_FORM_NAME: 'TourForm',
	
	MOVING_DATE_NAME: 'MovingDate',
	
	MOVING_TIME_NAME: 'MovingTime',

	
	addTravelTimesOnTransportPage:
	function addTravelTimesOnTransportPage(aDocument) {
		try {
			if (!pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_FEATURES_ENABLED)) {
				return;
			}
	
			if (!pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_ADD_TRAVEL_TIMES)) {
				return;
			}

			var fromLocaleId = aDocument.getElementsByName('FromLocaleID').item(0);
			
			pm_Logger.debug('From locale ID='+ fromLocaleId.value);
			
			if (fromLocaleId.value == '0') {
				pm_Charts.PrepareCities(aDocument);
				pm_TravelRoutes.initRouteDataByCities();
				this.autoSelectTransportStartCity(aDocument);
			}
			else {
				this.addTravelTimesForTransport(aDocument);
				this.autoSelectTransportDateAndTime(aDocument);
			}
		}
		catch (err) {
			pm_Logger.logError(err);
		}
	},	
	
	
	addTravelTimesForTransport:
	function addTravelTimesForTransport(aDocument) {
		try {
			var fromLocaleId = aDocument.getElementsByName(this.FROM_LOCALE_ID_NAME).item(0);
			var toLocaleId = aDocument.getElementsByName(this.TO_LOCALE_ID_NAME).item(0);
			
			if (fromLocaleId && fromLocaleId.options) {
				var opt1 = fromLocaleId.options.item(fromLocaleId.selectedIndex);
	
				pm_Logger.debug('opt='+ opt1.text);
				
				var res1 = opt1.text.match(this.CITY_NAME_REGEXP);
				
				if (res1) {
					var city1 = res1[1];
	
					for (var i = toLocaleId.options.length - 1; i >=0; i--) {
						var opt2 = toLocaleId.options.item(i);
						
						var res2 = opt2.text.match(this.CITY_NAME_REGEXP);
						
						if (res2) {
							var city2 = res2[1];
							
							pm_Logger.debug(city1 +' - '+ city2);
	
							city1Id = pm_Charts.GetCityIdByName(city1);
							city2Id = pm_Charts.GetCityIdByName(city2);
	
							pm_Logger.debug(city1Id +' - '+ city2Id);
	
							if (city1Id && city2Id) {
								var hours = this.getTravelTimeByCities(city1Id, city2Id);
								
								pm_Logger.debug('hours='+ hours);
								
								if (hours) {
									opt2.appendChild(aDocument.createTextNode('\u00a0('+ hours +'h)'));
								}
							}
						}
					}
				}
			}
		}
		catch (err) {
			pm_Logger.logError(err);
		}
	},
	
	
	addRouteInfoOnCityPage:
	function addRouteInfoOnCityPage(aDocument) {
		try {
			if (!pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_FEATURES_ENABLED)) {
				return;
			}

			var menu13 = aDocument.getElementById('Menu13');
			var aNodes = menu13.getElementsByTagName('a');
			
			pm_Logger.debug('nodes='+ aNodes.length);
			
			this.replaceWithLocalizedRouteName(aDocument, aNodes);

		}
		catch (err) {
			pm_Logger.logError(err);
		}		
	},


	replaceWithLocalizedRouteNameOnPage:
	function replaceWithLocalizedRouteNameOnPage(aDocument) {
		try {
			if (!pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_FEATURES_ENABLED)) {
				return;
			}

			var snapshotNodes = pm_XPathOrderedSnapshot(this.LOCALE_XPATH, aDocument);
			
			pm_Logger.debug('nodes='+ snapshotNodes.snapshotLength);
			
			this.replaceWithLocalizedRouteName(aDocument, snapshotNodes);

		}
		catch (err) {
			pm_Logger.logError(err);
		}		
	},


	addLocalizedRouteNameToTitle:
	function addLocalizedRouteNameToTitle(aDocument, aNodes) {
		try {
			var addTravelTime = pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_ADD_TRAVEL_TIMES);
			var addLocalizedRouteName = pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_ADD_LOCALIZED_ROUTE_NAME);
		
			if (addLocalizedRouteName) {
				if (!pm_Charts.PrepareCities(aDocument)) {
					pm_Logger.debug('City information not available');
					addLocalizedRouteName = false;
				}
			}
			
			if (!addTravelTime && !addLocalizedRouteName) {
				return;
			}

			for (var i = (aNodes.length || aNodes.snapshotLength) - 1; i >= 0; i--) {
				var node = aNodes.length ? aNodes.item(i) : aNodes.snapshotItem(i);
					
				if (addLocalizedRouteName) {
					var matchResult = node.search.match(this.LOCALE_ID_REGEXP);
	
					if (matchResult) {
						var localeId = matchResult[1];

						var routeData = pm_TravelRoutes.ROUTE_DATA_BY_LOCALE_ID[localeId];
			
						if (routeData) {
							var routeText = this.getLocalizedRouteName(routeData);
							
							if (addTravelTime) {
								routeText = routeText + ' ('+ routeData.travelTime +'h)';
							}
							
							if (node.title) {
								node.title = routeText +': '+ node.title;
							}							
							else {
								node.title = routeText;
							}
						}						
					}
				}
				else if (addTravelTime) {
					if (!this.isTravelTimeEmbedded(node.textContent)) {
						var hours = this.getTravelTimeByRouteLink(node);
						
						if (hours) {
							node.appendChild(aDocument.createTextNode('\u00a0('+ hours +'h)'));
						}
					}
				}
			}
		}		
		catch (err) {
			pm_Logger.logError(err);
		}		
	},


	replaceWithLocalizedRouteName:
	function replaceWithLocalizedRouteName(aDocument, aNodes) 
	{
		try {
			var addTravelTime = pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_ADD_TRAVEL_TIMES);
			var addLocalizedRouteName = pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_ADD_LOCALIZED_ROUTE_NAME);
		
			if (addLocalizedRouteName) {
				if (!pm_Charts.PrepareCities(aDocument)) {
					pm_Logger.debug('City information not available');
					addLocalizedRouteName = false;
				}
			}
			
			if (!addTravelTime && !addLocalizedRouteName) {
				return;
			}

			for (var i = (aNodes.length || aNodes.snapshotLength) - 1; i >= 0; i--) {
				var node = aNodes.length ? aNodes.item(i) : aNodes.snapshotItem(i);
				
				if (addLocalizedRouteName) {
					var matchResult = node.search.match(this.LOCALE_ID_REGEXP);
	
					if (matchResult) {
						var localeId = matchResult[1];

						var routeData = pm_TravelRoutes.ROUTE_DATA_BY_LOCALE_ID[localeId];
			
						if (routeData) {
							var span1 = this.getLocalizedRouteNameOnSpan(aDocument, routeData);
							
							if (span1) {
								if (addTravelTime) {
									var travelTimeText = aDocument.createTextNode(
										' ('+ routeData.travelTime +'h)');
		
									span1.appendChild(travelTimeText);
								}
								
								node.innerHTML = '';
								node.appendChild(span1);
							}
						}						
					}
				}
				else if (addTravelTime) {
					if (!this.isTravelTimeEmbedded(node.textContent)) {
						var hours = this.getTravelTimeByRouteLink(node);
						
						if (hours) {
							node.appendChild(aDocument.createTextNode('\u00a0('+ hours +'h)'));
						}
					}
				}
			}
		}
		catch (err) {
			pm_Logger.logError(err);
		}		
	},
	

	addRouteInfoOnHighwayLocalePage:
	function addRouteInfoOnHighwayLocalePage(aDocument) {
		try {
			var matchResult = aDocument.location.search.match(this.LOCALE_ID_REGEXP);

			if (matchResult) {
				if (!pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_FEATURES_ENABLED)) {
					return;
				}
	
				var addTravelTime = pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_ADD_TRAVEL_TIMES);
				var addLocalizedRouteName = pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_ADD_LOCALIZED_ROUTE_NAME);
			
				if (!addTravelTime && !addLocalizedRouteName) {
					return;
				}

				if (!pm_Charts.PrepareCities(aDocument)) {
					pm_Logger.debug('City information not available');
					return;
				}

				pm_Logger.debug('localeID='+ matchResult[1]);
						
				var localeId = parseInt(matchResult[1]);
				var routeData = pm_TravelRoutes.ROUTE_DATA_BY_LOCALE_ID[localeId];
	
				if (routeData) {
					var targetNode = pm_XPathFirstNode(this.ROUTE_NAME_TARGET_ON_HIGHWAY_LOCALE_PAGE_XPATH, aDocument);
						
					if (targetNode) {
						var pn = targetNode.parentNode;

						if (addLocalizedRouteName) {
							var span1 = this.getLocalizedRouteNameOnSpan(
								aDocument, routeData);
							
							if (span1) {
								if (addTravelTime) {
									var travelTimeText = aDocument.createTextNode(
										' ('+ routeData.travelTime +'h)');
		
									span1.appendChild(travelTimeText);
								}

								var br1 = aDocument.createElement('br');
								pn.insertBefore(br1, targetNode);
								pn.insertBefore(span1, targetNode);
							}
						}
						else {
							if (addTravelTime) {
								var travelTimeText = aDocument.createTextNode(
									' ('+ routeData.travelTime +'h)');
	
								pn.insertBefore(travelTimeText, targetNode);
							}
						}						
					}
					else {
						pm_Logger.debug('Unable to locate the target node');
						return;
					}
					
				}
				else { 
					pm_Logger.debug('Not a route');
					return; // Not a route (or a new route).
				}
			}
			else {
				pm_Logger.debug('No locale ID available');
				return; // No locale ID available.
			}
		}
		catch (err) {
			pm_Logger.logError(err);
		}		
	},


	getLocalizedRouteName:
	function getLocalizedRouteName(routeData) {
		try {
			var fromCityName = pm_Charts.GetCityNameById(routeData.from);
			var toCityName = pm_Charts.GetCityNameById(routeData.to);
			
			if (fromCityName && toCityName) {
				var routeName = fromCityName +' - '+ toCityName;
				return routeName;
			}			
		}
		catch (err) {
			pm_Logger.logError(err);
		}
		
		return undefined;
	},


	getLocalizedRouteNameOnSpan: 
	function getLocalizedRouteNameOnSpan(aDocument, routeData) {
		try {
			var routeName = this.getLocalizedRouteName(routeData);
			
			if (routeName) {
				var span1 = aDocument.createElement('span');
				span1.appendChild(aDocument.createTextNode(routeName));
				span1.className = 'popomungo_route';
				
				return span1;
			}		
		}
		catch (err) {
			pm_Logger.logError(err);
		}
		
		return null;
	},
		
	
	isTravelTimeEmbedded:
	function isTravelTimeEmbedded(routeName) {
		try {
			var res = routeName.match(this.HOURS_ON_ROUTE_NAME);

			if (res) {
				return res[1];
			}			
		}
		catch (err) {
			pm_Logger.logError(err);
		}
		
		return false;	
	},
	

	getTravelTimeByCities: function getTravelTimeByCities(city1Id, city2Id) {
		try {
			var routeData = pm_TravelRoutes.ROUTE_DATA_BY_CITIES[city1Id][city2Id];

			return routeData ? routeData.travelTime : undefined;
		}
		catch (err) {
			pm_Logger.logError(err);
		}

		return undefined;			
	},
	
	
	getTravelTimeByRouteLink: 
	function getTravelTimeByRouteLink(node) {
		try {
			var matchResult = node.href.match(this.LOCALE_ID_REGEXP);
			
			if (matchResult) {
				var localeId = matchResult[1];
				
				var routeData = pm_TravelRoutes.ROUTE_DATA_BY_LOCALE_ID[localeId];
				
				return routeData ? routeData.travelTime : undefined;
			}			
		}	
		catch (err) {
			pm_Logger.logError(err);
		}
		
		return undefined;
	},
	
	
	autoSelectTransportStartCity:
	function autoSelectTransportStartCity(aDocument) {
		try {
			if (!pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_FEATURES_ENABLED)) {
				return;
			}
			
			if (!pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_AUTO_SELECT_TRANSPORT_START_CITY)) {
				return;
			}
			
			var lastRow = pm_XPathFirstNode(this.LAST_TRANSPORT_ROW_XPATH, aDocument);

			if (!lastRow) {
				return;
			}

			pm_Logger.debug(lastRow);

			var td1 = lastRow.cells.item(0);
			
			var dataMatch = td1.textContent.match(this.DATE_TIME_AND_CITY_NAME_ON_TRANSPORT_ROW_REGEXP);
			pm_Logger.debug(td1.textContent);
			
			if (dataMatch && dataMatch.length > 1) {
				//var endDate = pm_Trim(dataMatch[1]);
				//var endTime = pm_Trim(dataMatch[2]);
				var city = pm_Trim(dataMatch[3]);
				
				pm_Logger.debug('city='+ city);
			
				var opt;
				var fromLocaleId = aDocument.getElementsByName(this.FROM_LOCALE_ID_NAME).item(0);
				
				for (var i = fromLocaleId.options.length - 1; i >= 0; i--) {
					opt = fromLocaleId.options.item(i);
					
					//pm_Logger.debug(i +': fromLocaleId.text='+ opt.text);

					if (opt.text.match(city)) {
						pm_Logger.debug('fromLocaleId.selectedIndex='+ i);
						
						fromLocaleId.selectedIndex = i;

						break;
					}
				}

				aDocument.getElementsByName(this.TO_LOCALE_ID_NAME).item(0).disabled = true;
				aDocument.getElementsByName(this.MOVING_DATE_NAME).item(0).disabled = true;
				aDocument.getElementsByName(this.MOVING_TIME_NAME).item(0).disabled = true;

/*
				var tourForm = aDocument.getElementsByName(this.TOUR_FORM_NAME).item(0);

				if (tourForm) {
					pm_Logger.debug('Submit '+ this.TOUR_FORM_NAME);
					tourForm.submit();
				}
*/
			}
		}
		catch (err) {
			pm_Logger.logError(err);
		}
	},
	
	
	autoSelectTransportDateAndTime:
	function autoSelectTransportDateAndTime(aDocument) {
		try {
			if (!pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_FEATURES_ENABLED)) {
				return;
			}
			
			if (!pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_AUTO_SELECT_TRANSPORT_START_TIME)) {
				return;
			}
			
			var lastRow = pm_XPathFirstNode(this.LAST_TRANSPORT_ROW_XPATH, aDocument);

			if (!lastRow) {
				return;
			}

			pm_Logger.debug(lastRow);

			var td1 = lastRow.cells.item(0);
			
			var dataMatch = td1.textContent.match(this.DATE_TIME_AND_CITY_NAME_ON_TRANSPORT_ROW_REGEXP);
			pm_Logger.debug(td1.textContent);
			
			if (dataMatch && dataMatch.length > 1) {
				var endDate = pm_Trim(dataMatch[1]);
				var endTime = pm_Trim(dataMatch[2]);
				//var city = pm_Trim(dataMatch[3]);
				
				pm_Logger.debug('endDate='+ endDate +', endTime='+ endTime);
			
				var opt;

				var movingDate = aDocument.getElementsByName(this.MOVING_DATE_NAME).item(0);
				
				for (var i = movingDate.options.length - 1; i >= 0; i--) {
					opt = movingDate.options.item(i);

					//pm_Logger.debug(i +': movingDate.text='+ opt.text);

					if (opt.text.match(endDate)) {
						pm_Logger.debug('movingDate.selectedIndex='+ i);
						
						movingDate.selectedIndex = i;

						break;
					}
				}					

				var movingTime = aDocument.getElementsByName(this.MOVING_TIME_NAME).item(0);
				
				for (var i = movingTime.options.length - 1; i >= 0; i--) {
					opt = movingTime.options.item(i);

					//pm_Logger.debug(i +': movingTime.text='+ opt.text);

					if (opt.text.match(endTime)) {
						pm_Logger.debug('movingTime.selectedIndex='+ i);
						
						movingTime.selectedIndex = i;

						break;
					}
				}					
			}
		}
		catch (err) {
			pm_Logger.logError(err);
		}
	},
	
	
	addCityNameToBusStationName: function addCityNameToBusStationName(aDocument) {
		try {
			if (!pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_FEATURES_ENABLED)) {
				return;
			}
			
			if (!pm_Prefs.isEnabled(pm_PrefKeys.TRAVELING_ADD_CITY_FOR_BUS_TERMINAL)) {
				return;
			}

			var linkNode = pm_XPathFirstNode(this.BUS_STATION_LINK_ON_CAREER_PAGE_XPATH, aDocument);
			
			if (linkNode) {
				var idMatch = linkNode.search.match(this.LOCALE_ID_REGEXP);
				
				if (!pm_Charts.PrepareCities(aDocument)) {
					return;
				}				
				
				if (idMatch) {
					var localeId = idMatch[1];
					
					var cityId = pm_BusTerminals.getCityByBusTerminal(localeId);
					
					if (cityId) {
						var city = pm_Charts.GetCityNameById(cityId); 
						
						if (city) {
							pm_Logger.debug("City="+ city);
							linkNode.appendChild(aDocument.createTextNode(", "+ city));
						}
						else {
							pm_Logger.debug("No city");
						}
					}
					else {
						pm_Logger.debug("No city ID.");
					}
				}
				else {
					pm_Logger.debug("Could not parse locale ID from URL.");
				}
			}
			else {
				pm_Logger.debug("No link node.");
			}
		}
		catch (err) {
			pm_Logger.logError(err);
		}
	}
};
